searchData={"items":[{"type":"module","title":"ExTermbox","doc":"Elixir bindings for the termbox library, providing a way to control the\nterminal and draw simple UIs.\n\nThis module provides the main user-facing API. It interacts with a C process\nvia an Elixir Port managed by the `ExTermbox.PortHandler` GenServer.","ref":"ExTermbox.html"},{"type":"module","title":"Usage - ExTermbox","doc":"1.  **Initialization:** Call `ExTermbox.init/1` to start the necessary processes.\n    This returns `{:ok, pid}` where `pid` is the handler process.\n    This must be called before any other functions.\n2.  **API Calls:** Use functions like `change_cell/6`, `clear/1`, etc., passing\n    the `pid` obtained from `init/1` as the first argument.\n3.  **Display:** Call `ExTermbox.present/1` (passing the `pid`) to flush the\n    back buffer to the terminal screen.\n4.  **Events:** Call `ExTermbox.poll_event/1` (passing the `pid`) to wait for\n    user input or resize events. These will be sent as messages to the process\n    that called `init/1` (the owner).\n5.  **Shutdown:** Call `ExTermbox.shutdown/1` (passing the `pid`) when finished\n    to clean up.","ref":"ExTermbox.html#module-usage"},{"type":"module","title":"Event Handling - ExTermbox","doc":"Events are delivered as messages in the format `{:termbox_event, event_map}`\nto the process that called `init/1`. See `ExTermbox.PortHandler` for details\non the event map structure. You typically need to call `poll_event/1` again\nafter receiving an event to wait for the next one.\n\nExample (`handle_info` in the process that called `init/1`):\n\n```elixir\ndef handle_info({:termbox_event, event}, state) do\n  # Assuming handler_pid is stored in state\n  handler_pid = state.handler_pid\n  IO.inspect(event, label: \"Received Termbox Event\")\n  # Request next event, passing the pid\n  ExTermbox.poll_event(handler_pid)\n  {:noreply, state}\nend\n```","ref":"ExTermbox.html#module-event-handling"},{"type":"function","title":"ExTermbox.change_cell/6","doc":"Changes the character, foreground, and background attributes of a specific cell.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `x`: The zero-based column index.\n  - `y`: The zero-based row index.\n  - `char`: The character (as a single-char string or integer codepoint).\n  - `fg`: The foreground attribute (e.g., `ExTermbox.Const.Color.RED`).\n  - `bg`: The background attribute (e.g., `ExTermbox.Const.Attribute.BOLD`).","ref":"ExTermbox.html#change_cell/6"},{"type":"function","title":"ExTermbox.clear/1","doc":"Clears the terminal buffer using the default colors.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#clear/1"},{"type":"function","title":"ExTermbox.debug_crash/1","doc":"[Debug] Causes the C helper process to exit immediately.\nFOR TESTING ONLY.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#debug_crash/1"},{"type":"function","title":"ExTermbox.debug_send_event/9","doc":"[Debug] Sends a synthetic event for testing purposes.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#debug_send_event/9"},{"type":"function","title":"ExTermbox.get_cell/3","doc":"Retrieves the content of a specific cell from the back buffer.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `x`: The zero-based column index.\n  - `y`: The zero-based row index.\n\nReturns `{:ok, cell_map}` or `{:error, reason}`.\nThe `cell_map` has keys `:x, :y, :char, :fg, :bg`.","ref":"ExTermbox.html#get_cell/3"},{"type":"function","title":"ExTermbox.height/1","doc":"Returns the height of the terminal.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#height/1"},{"type":"function","title":"ExTermbox.init/1","doc":"Initializes the termbox library.\n\nStarts the PortHandler GenServer.\nReturns `{:ok, pid}` on success, where `pid` is the PortHandler process ID,\nor `{:error, reason}` on failure.\n\nOptions:\n  - `name`: Registers the GenServer under a specific name.\n  - `owner`: Specifies the owner process (defaults to `self()`).","ref":"ExTermbox.html#init/1"},{"type":"type","title":"ExTermbox.pid_or_name/0","doc":"","ref":"ExTermbox.html#t:pid_or_name/0"},{"type":"function","title":"ExTermbox.poll_event/1","doc":"Requests the PortHandler to poll for the next event.\nThis is typically called after handling a previous event.\nThe event will be delivered asynchronously to the owning process.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#poll_event/1"},{"type":"function","title":"ExTermbox.present/1","doc":"Synchronizes the back buffer with the terminal screen.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#present/1"},{"type":"function","title":"ExTermbox.print/6","doc":"Prints a string at the specified position with given attributes.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `x`: The zero-based column index for the start of the string.\n  - `y`: The zero-based row index for the start of the string.\n  - `fg`: The foreground attribute (e.g., `ExTermbox.Const.Color.RED`).\n  - `bg`: The background attribute (e.g., `ExTermbox.Const.Attribute.BOLD`).\n  - `string`: The string to print.","ref":"ExTermbox.html#print/6"},{"type":"function","title":"ExTermbox.select_input_mode/2","doc":"Selects the input mode.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `mode`: An atom representing the input mode (e.g., `ExTermbox.Const.Input.ESC`).","ref":"ExTermbox.html#select_input_mode/2"},{"type":"function","title":"ExTermbox.set_clear_attributes/3","doc":"Sets the default foreground and background attributes used by `clear/1`.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `fg`: The foreground attribute.\n  - `bg`: The background attribute.","ref":"ExTermbox.html#set_clear_attributes/3"},{"type":"function","title":"ExTermbox.set_cursor/3","doc":"Sets the cursor position. Use `(-1, -1)` to hide the cursor.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `x`: The zero-based column index, or -1 to hide.\n  - `y`: The zero-based row index, or -1 to hide.","ref":"ExTermbox.html#set_cursor/3"},{"type":"function","title":"ExTermbox.set_output_mode/2","doc":"Selects the output mode.\n\nRequires the PID or registered name of the PortHandler process.\n\nArguments:\n  - `pid_or_name`: The PID or registered name of the PortHandler.\n  - `mode`: An atom representing the output mode (e.g., `ExTermbox.Const.OutputMode.C256`).","ref":"ExTermbox.html#set_output_mode/2"},{"type":"function","title":"ExTermbox.shutdown/1","doc":"Shuts down the termbox library and stops the PortHandler GenServer.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#shutdown/1"},{"type":"function","title":"ExTermbox.start_link/1","doc":"","ref":"ExTermbox.html#start_link/1"},{"type":"function","title":"ExTermbox.width/1","doc":"Returns the width of the terminal.\n\nRequires the PID or registered name of the PortHandler process.","ref":"ExTermbox.html#width/1"},{"type":"module","title":"ExTermbox.Buffer","doc":"Handles buffering of incoming data chunks from the Unix Domain Socket.\nExtracts complete lines terminated by newline characters.","ref":"ExTermbox.Buffer.html"},{"type":"function","title":"ExTermbox.Buffer.process/2","doc":"Processes an incoming data chunk, appending it to the buffer and extracting lines.\n\nReturns `{:lines, lines, remaining_buffer}` or `{:incomplete, buffer}` where \n`lines` is a list of complete, trimmed lines found in the combined buffer, \nand `remaining_buffer` / `buffer` is the content after the last newline or the\nentire buffer if no newline was found.","ref":"ExTermbox.Buffer.html#process/2"},{"type":"module","title":"ExTermbox.Cell","doc":"Represents a termbox cell, a character at a position, along with the cell's\nbackground and foreground colors.","ref":"ExTermbox.Cell.html"},{"type":"function","title":"ExTermbox.Cell.empty/0","doc":"","ref":"ExTermbox.Cell.html#empty/0"},{"type":"type","title":"ExTermbox.Cell.t/0","doc":"","ref":"ExTermbox.Cell.html#t:t/0"},{"type":"module","title":"ExTermbox.Constants","doc":"Defines constants from the termbox library. These can be used e.g. to set a\nformatting attributes or to identify keys passed in an event.","ref":"ExTermbox.Constants.html"},{"type":"type","title":"ExTermbox.Constants.attribute/0","doc":"","ref":"ExTermbox.Constants.html#t:attribute/0"},{"type":"function","title":"ExTermbox.Constants.attribute/1","doc":"Retrieves an attribute constant by name","ref":"ExTermbox.Constants.html#attribute/1"},{"type":"function","title":"Examples - ExTermbox.Constants.attribute/1","doc":"iex> attribute(:bold)\n    0x0100\n    iex> attribute(:underline)\n    0x0200","ref":"ExTermbox.Constants.html#attribute/1-examples"},{"type":"function","title":"ExTermbox.Constants.attributes/0","doc":"Retrieves the mapping of attribute constants.","ref":"ExTermbox.Constants.html#attributes/0"},{"type":"type","title":"ExTermbox.Constants.color/0","doc":"","ref":"ExTermbox.Constants.html#t:color/0"},{"type":"function","title":"ExTermbox.Constants.color/1","doc":"Retrieves a color constant by name","ref":"ExTermbox.Constants.html#color/1"},{"type":"function","title":"Examples - ExTermbox.Constants.color/1","doc":"iex> color(:red)\n    0x02\n    iex> color(:blue)\n    0x05","ref":"ExTermbox.Constants.html#color/1-examples"},{"type":"function","title":"ExTermbox.Constants.colors/0","doc":"Retrieves the mapping of color constants.","ref":"ExTermbox.Constants.html#colors/0"},{"type":"type","title":"ExTermbox.Constants.constant/0","doc":"","ref":"ExTermbox.Constants.html#t:constant/0"},{"type":"type","title":"ExTermbox.Constants.error_code/0","doc":"","ref":"ExTermbox.Constants.html#t:error_code/0"},{"type":"function","title":"ExTermbox.Constants.error_code/1","doc":"Retrieves an error code constant by name","ref":"ExTermbox.Constants.html#error_code/1"},{"type":"function","title":"Examples - ExTermbox.Constants.error_code/1","doc":"iex> error_code(:unsupported_terminal)\n    -1\n    iex> error_code(:pipe_trap_error)\n    -3","ref":"ExTermbox.Constants.html#error_code/1-examples"},{"type":"function","title":"ExTermbox.Constants.error_codes/0","doc":"Retrieves the mapping of error code constants.","ref":"ExTermbox.Constants.html#error_codes/0"},{"type":"type","title":"ExTermbox.Constants.event_type/0","doc":"","ref":"ExTermbox.Constants.html#t:event_type/0"},{"type":"function","title":"ExTermbox.Constants.event_type/1","doc":"Retrieves an event type constant by name","ref":"ExTermbox.Constants.html#event_type/1"},{"type":"function","title":"Examples - ExTermbox.Constants.event_type/1","doc":"iex> event_type(:key)\n    0x01\n    iex> event_type(:resize)\n    0x02\n    iex> event_type(:mouse)\n    0x03","ref":"ExTermbox.Constants.html#event_type/1-examples"},{"type":"function","title":"ExTermbox.Constants.event_types/0","doc":"Retrieves the mapping of event type constants.","ref":"ExTermbox.Constants.html#event_types/0"},{"type":"function","title":"ExTermbox.Constants.hide_cursor/0","doc":"Retrieves the hide cursor constant.","ref":"ExTermbox.Constants.html#hide_cursor/0"},{"type":"function","title":"Examples - ExTermbox.Constants.hide_cursor/0","doc":"iex> hide_cursor()\n    -1","ref":"ExTermbox.Constants.html#hide_cursor/0-examples"},{"type":"type","title":"ExTermbox.Constants.hide_cursor/0","doc":"","ref":"ExTermbox.Constants.html#t:hide_cursor/0"},{"type":"type","title":"ExTermbox.Constants.input_mode/0","doc":"","ref":"ExTermbox.Constants.html#t:input_mode/0"},{"type":"function","title":"ExTermbox.Constants.input_mode/1","doc":"Retrieves an input mode constant by name","ref":"ExTermbox.Constants.html#input_mode/1"},{"type":"function","title":"Examples - ExTermbox.Constants.input_mode/1","doc":"iex> input_mode(:esc)\n    1\n    iex> input_mode(:mouse)\n    4","ref":"ExTermbox.Constants.html#input_mode/1-examples"},{"type":"function","title":"ExTermbox.Constants.input_modes/0","doc":"Retrieves the mapping of input mode constants.","ref":"ExTermbox.Constants.html#input_modes/0"},{"type":"type","title":"ExTermbox.Constants.key/0","doc":"","ref":"ExTermbox.Constants.html#t:key/0"},{"type":"function","title":"ExTermbox.Constants.key/1","doc":"Retrieves a key constant by name","ref":"ExTermbox.Constants.html#key/1"},{"type":"function","title":"Examples - ExTermbox.Constants.key/1","doc":"iex> key(:esc)\n    0x1B\n    iex> key(:space)\n    0x20","ref":"ExTermbox.Constants.html#key/1-examples"},{"type":"function","title":"ExTermbox.Constants.keys/0","doc":"Retrieves the mapping of key constants for use with termbox.\n\nThese are based on terminfo constants. Note that there's some overlap\nof terminfo values. For example, it's not possible to distinguish between\n` ` and `ctrl-h`.","ref":"ExTermbox.Constants.html#keys/0"},{"type":"function","title":"ExTermbox.Constants.mod/1","doc":"Retrieves a modifier constant by name","ref":"ExTermbox.Constants.html#mod/1"},{"type":"function","title":"Examples - ExTermbox.Constants.mod/1","doc":"iex> mod(:alt)\n    1\n    iex> mod(:none)\n    0","ref":"ExTermbox.Constants.html#mod/1-examples"},{"type":"type","title":"ExTermbox.Constants.modifier/0","doc":"","ref":"ExTermbox.Constants.html#t:modifier/0"},{"type":"function","title":"ExTermbox.Constants.modifiers/0","doc":"Retrieves the mapping of modifier constants.","ref":"ExTermbox.Constants.html#modifiers/0"},{"type":"type","title":"ExTermbox.Constants.output_mode/0","doc":"","ref":"ExTermbox.Constants.html#t:output_mode/0"},{"type":"function","title":"ExTermbox.Constants.output_mode/1","doc":"Retrieves an output mode constant by name","ref":"ExTermbox.Constants.html#output_mode/1"},{"type":"function","title":"Examples - ExTermbox.Constants.output_mode/1","doc":"iex> output_mode(:normal)\n    1\n    iex> output_mode(:term_256)\n    2","ref":"ExTermbox.Constants.html#output_mode/1-examples"},{"type":"function","title":"ExTermbox.Constants.output_modes/0","doc":"Retrieves the mapping of output mode constants.","ref":"ExTermbox.Constants.html#output_modes/0"},{"type":"function","title":"ExTermbox.Constants.resolve_color/1","doc":"Resolves a color atom (e.g., :red) or integer to its integer value.\nReturns the default color value if the input is invalid or not found.","ref":"ExTermbox.Constants.html#resolve_color/1"},{"type":"module","title":"ExTermbox.Event","doc":"Represents a termbox event. This can be a key press, click, or window resize.","ref":"ExTermbox.Event.html"},{"type":"module","title":"ExTermbox.EventManager","doc":"This module implements an event manager that notifies subscribers of\nkeyboard, mouse and resize events received from the termbox API.\n\nInternally, the event manager is managing a NIF-based polling routine and\nfanning out polled events to subscribers. It works likes this:\n\n  1. The `ExTermbox.Bindings.start_polling/1` NIF is called with the event\n     manager's pid.\n  2. The NIF creates a background thread for the blocking polling routine and\n     immediately returns with a resource representing a handle for the thread.\n  3. When the polling routine receives an event (e.g., a keypress), it sends\n     a message to the event manager with the event data, and then continues\n     polling for the next event.\n  4. The event manager receives event data from the background thread and\n     notifies all of its subscribers of the event. Steps 3 and 4 are repeated\n     for each event.\n  5. When the event manager is terminated, `ExTermbox.Bindings.stop_polling/0`\n     is called to stop polling and terminate the background thread.\n\nExample Usage:\n\n    def event_loop do\n      receive do\n        {:event, %Event{ch: ?q} = event} ->\n          Bindings.shutdown()\n        {:event, %Event{} = event} ->\n          # handle the event and wait for another...\n          event_loop()\n      end\n    end\n\n    {:ok, pid} = EventManager.start_link()\n    :ok = EventManager.subscribe(self())\n    event_loop()","ref":"ExTermbox.EventManager.html"},{"type":"function","title":"ExTermbox.EventManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExTermbox.EventManager.html#child_spec/1"},{"type":"function","title":"ExTermbox.EventManager.start_link/1","doc":"Starts an event manager process linked to the current process.\n\nRunning multiple instances of the event manager process simultaneously is\ndiscouraged, as it could crash the NIF or cause unexpected behavior. By\ndefault, the process is registered with a fixed name to prevent this.","ref":"ExTermbox.EventManager.html#start_link/1"},{"type":"function","title":"ExTermbox.EventManager.stop/1","doc":"","ref":"ExTermbox.EventManager.html#stop/1"},{"type":"function","title":"ExTermbox.EventManager.subscribe/2","doc":"Subscribes the given subscriber pid to future event notifications.","ref":"ExTermbox.EventManager.html#subscribe/2"},{"type":"module","title":"ExTermbox.InitHandler","doc":"Handles the initialization sequence within the PortHandler GenServer.\nThis module encapsulates the logic for parsing port output, managing\ntimeouts, and establishing the UDS connection.","ref":"ExTermbox.InitHandler.html"},{"type":"function","title":"ExTermbox.InitHandler.handle_continue_attempt_connect/2","doc":"Called by PortHandler via :continue after the UDS connect delay.\nChecks socket existence and attempts connection.","ref":"ExTermbox.InitHandler.html#handle_continue_attempt_connect/2"},{"type":"function","title":"ExTermbox.InitHandler.handle_port_data_init/2","doc":"Called by PortHandler when it receives data from the port during initialization.\nParses the data to find the socket path.\nTransitions PortHandler state to :connecting_socket or stops on error.","ref":"ExTermbox.InitHandler.html#handle_port_data_init/2"},{"type":"function","title":"ExTermbox.InitHandler.handle_port_exit_init/2","doc":"Called by PortHandler when the OS port exits during initialization.\nReplies error to the original caller and stops the PortHandler.","ref":"ExTermbox.InitHandler.html#handle_port_exit_init/2"},{"type":"function","title":"ExTermbox.InitHandler.handle_socket_connect_failure/4","doc":"Called by PortHandler when the :gen_tcp socket connection fails.\nReplies error to the original caller and stops the PortHandler.","ref":"ExTermbox.InitHandler.html#handle_socket_connect_failure/4"},{"type":"function","title":"ExTermbox.InitHandler.handle_socket_connect_success/2","doc":"Called by PortHandler when the :gen_tcp socket connection succeeds.\nReplies :ok to the original caller and transitions PortHandler state.","ref":"ExTermbox.InitHandler.html#handle_socket_connect_success/2"},{"type":"function","title":"ExTermbox.InitHandler.handle_timeout_init/2","doc":"Called by PortHandler on timeout during initialization stages.\nReplies error to the original caller and stops the PortHandler.","ref":"ExTermbox.InitHandler.html#handle_timeout_init/2"},{"type":"module","title":"ExTermbox.Initializer","doc":"Handles the initial setup and supervision of Termbox resources.","ref":"ExTermbox.Initializer.html"},{"type":"function","title":"ExTermbox.Initializer.handle_pty_line/2","doc":"Handles a single line received via PTY during the initialization handshake.\n\nTakes the line and the current PortHandler state.\n\nReturns:\n- `{:cont, new_state}`: If the handshake should continue to the next stage.\n- `{:socket_ready, socket, new_state}`: If the handshake is complete and the socket is connected.\n- `{:halt, {:stop, reason, new_state}}`: If the handshake failed irrecoverably.","ref":"ExTermbox.Initializer.html#handle_pty_line/2"},{"type":"module","title":"ExTermbox.PortHandler","doc":"GenServer responsible for managing the Erlang Port connected to the C helper process\nand the Unix Domain Socket (UDS) communication.","ref":"ExTermbox.PortHandler.html"},{"type":"function","title":"ExTermbox.PortHandler.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExTermbox.PortHandler.html#child_spec/1"},{"type":"function","title":"ExTermbox.PortHandler.start_link/1","doc":"","ref":"ExTermbox.PortHandler.html#start_link/1"},{"type":"module","title":"ExTermbox.PortHandler.CallHandler","doc":"Handles `handle_call/3` callbacks and logic for `ExTermbox.PortHandler`.","ref":"ExTermbox.PortHandler.CallHandler.html"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_call_when_not_connected/3","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_call_when_not_connected/3"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_get_cell/4","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_get_cell/4"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_height/3","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_height/3"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_request_port_close/1","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_request_port_close/1"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_set_cursor/4","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_set_cursor/4"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_simple_command/4","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_simple_command/4"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_trigger_poll_event/1","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_trigger_poll_event/1"},{"type":"function","title":"ExTermbox.PortHandler.CallHandler.handle_width/3","doc":"","ref":"ExTermbox.PortHandler.CallHandler.html#handle_width/3"},{"type":"module","title":"ExTermbox.PortHandler.InitHandler","doc":"Handles the initialization logic (`init/1` callback and related messages) for `ExTermbox.PortHandler`.","ref":"ExTermbox.PortHandler.InitHandler.html"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.do_connect_socket/3","doc":"","ref":"ExTermbox.PortHandler.InitHandler.html#do_connect_socket/3"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.handle_init_timeout/3","doc":"Handles a timeout during the initialization phase.\nReplies error to the original `start_link` caller and prepares state for stop.","ref":"ExTermbox.PortHandler.InitHandler.html#handle_init_timeout/3"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.handle_port_data_init/2","doc":"Handles incoming data packets received via the Port during the initial phase.\nIt buffers data and attempts to parse the 'OK  ' line.\nOnce the path is found, it initiates the UDS connection attempt.\n\nReturns:\n- `{:noreply, state_updates_map, timeout | nil}`: If more data is needed or connection started.\n- `{:stop, reason, state_updates_map}`: If parsing fails or connection initiation fails.","ref":"ExTermbox.PortHandler.InitHandler.html#handle_port_data_init/2"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.init_stage_connected/0","doc":"","ref":"ExTermbox.PortHandler.InitHandler.html#init_stage_connected/0"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.init_stage_connecting_socket/0","doc":"","ref":"ExTermbox.PortHandler.InitHandler.html#init_stage_connecting_socket/0"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.init_stage_init_failed/0","doc":"","ref":"ExTermbox.PortHandler.InitHandler.html#init_stage_init_failed/0"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.init_stage_waiting_port_data/0","doc":"","ref":"ExTermbox.PortHandler.InitHandler.html#init_stage_waiting_port_data/0"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.internal_init_timeout/0","doc":"","ref":"ExTermbox.PortHandler.InitHandler.html#internal_init_timeout/0"},{"type":"function","title":"ExTermbox.PortHandler.InitHandler.start_port_and_init/1","doc":"The main entry point for PortHandler initialization.\nFinds the C helper, spawns the Port, sends the initial trigger,\nand returns the initial state or stop reason.","ref":"ExTermbox.PortHandler.InitHandler.html#start_port_and_init/1"},{"type":"module","title":"ExTermbox.PortHandler.PortExitHandler","doc":"Handles Port exit (`{:EXIT, port, reason}`) messages for `ExTermbox.PortHandler`.","ref":"ExTermbox.PortHandler.PortExitHandler.html"},{"type":"function","title":"ExTermbox.PortHandler.PortExitHandler.handle_port_exit/2","doc":"Handles the unexpected termination of the underlying C port process.\n\nIt logs the error, updates the state to reflect the failure, notifies the owner,\nattempts to reply with an error to any pending caller, cleans up the socket,\nand returns a `:stop` tuple for the PortHandler GenServer.","ref":"ExTermbox.PortHandler.PortExitHandler.html#handle_port_exit/2"},{"type":"module","title":"ExTermbox.PortHandler.SocketHandler","doc":"Handles Unix Domain Socket interactions (`:tcp_*` messages) for `ExTermbox.PortHandler`.","ref":"ExTermbox.PortHandler.SocketHandler.html"},{"type":"function","title":"ExTermbox.PortHandler.SocketHandler.handle_command_timeout/2","doc":"","ref":"ExTermbox.PortHandler.SocketHandler.html#handle_command_timeout/2"},{"type":"function","title":"ExTermbox.PortHandler.SocketHandler.handle_socket_closed/2","doc":"","ref":"ExTermbox.PortHandler.SocketHandler.html#handle_socket_closed/2"},{"type":"function","title":"ExTermbox.PortHandler.SocketHandler.handle_socket_data/4","doc":"","ref":"ExTermbox.PortHandler.SocketHandler.html#handle_socket_data/4"},{"type":"function","title":"ExTermbox.PortHandler.SocketHandler.handle_socket_error/3","doc":"","ref":"ExTermbox.PortHandler.SocketHandler.html#handle_socket_error/3"},{"type":"module","title":"ExTermbox.PortHandler.State","doc":"Defines the state structure for the ExTermbox.PortHandler GenServer.","ref":"ExTermbox.PortHandler.State.html"},{"type":"type","title":"ExTermbox.PortHandler.State.t/0","doc":"","ref":"ExTermbox.PortHandler.State.html#t:t/0"},{"type":"module","title":"ExTermbox.PortHandler.TerminationHandler","doc":"Handles the `terminate/2` callback for `ExTermbox.PortHandler`.","ref":"ExTermbox.PortHandler.TerminationHandler.html"},{"type":"function","title":"ExTermbox.PortHandler.TerminationHandler.terminate/2","doc":"Handles the termination of the PortHandler GenServer.\nEnsures resources like the port and socket are cleaned up.\nNotifies the owner and replies to pending calls on abnormal termination.","ref":"ExTermbox.PortHandler.TerminationHandler.html#terminate/2"},{"type":"module","title":"ExTermbox.Position","doc":"Represents a position on the screen by encoding a pair of cartesian\ncoordinates. The origin is the top-left-most character on the screen\n`(0, 0)`, while x and y increase from left to right and top to bottom,\nrespectively.","ref":"ExTermbox.Position.html"},{"type":"type","title":"ExTermbox.Position.t/0","doc":"","ref":"ExTermbox.Position.html#t:t/0"},{"type":"function","title":"ExTermbox.Position.translate/3","doc":"Translates (shifts) a position by some delta x and y.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate/3"},{"type":"function","title":"Examples - ExTermbox.Position.translate/3","doc":"iex> translate(%Position{x: 0, y: 0}, 1, 2)\n    %Position{x: 1, y: 2}\n    iex> translate(%Position{x: 10, y: 0}, -1, 0)\n    %Position{x: 9, y: 0}","ref":"ExTermbox.Position.html#translate/3-examples"},{"type":"function","title":"ExTermbox.Position.translate_x/2","doc":"Translates a position by a delta x.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate_x/2"},{"type":"function","title":"Examples - ExTermbox.Position.translate_x/2","doc":"iex> translate_x(%Position{x: 0, y: 0}, 2)\n    %Position{x: 2, y: 0}\n    iex> translate_x(%Position{x: 2, y: 0}, -1)\n    %Position{x: 1, y: 0}","ref":"ExTermbox.Position.html#translate_x/2-examples"},{"type":"function","title":"ExTermbox.Position.translate_y/2","doc":"Translates a position by a delta y.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate_y/2"},{"type":"function","title":"Examples - ExTermbox.Position.translate_y/2","doc":"iex> translate_y(%Position{x: 0, y: 0}, 2)\n    %Position{x: 0, y: 2}\n    iex> translate_y(%Position{x: 0, y: 2}, -1)\n    %Position{x: 0, y: 1}","ref":"ExTermbox.Position.html#translate_y/2-examples"},{"type":"module","title":"ExTermbox.ProcessManager","doc":"Manages the lifecycle of the C helper process and the PortHandler GenServer.\n\nProvides functions to:\n- Spawn the Port process (`spawn_port_process/2`)\n- Connect to the socket exposed by the C helper (`connect_socket/1`)\n- Send commands to the C helper via the Port (`send_command_to_port/2`)\n- Manage the socket connection (`manage_socket_connection/1`)","ref":"ExTermbox.ProcessManager.html"},{"type":"function","title":"ExTermbox.ProcessManager.close_port/1","doc":"Uses Port.close to close the port.","ref":"ExTermbox.ProcessManager.html#close_port/1"},{"type":"function","title":"ExTermbox.ProcessManager.close_socket/1","doc":"Uses :gen_tcp.close for closing the UDS created via :gen_tcp.","ref":"ExTermbox.ProcessManager.html#close_socket/1"},{"type":"function","title":"ExTermbox.ProcessManager.command_port/2","doc":"Uses Port.command to send data to the port process.","ref":"ExTermbox.ProcessManager.html#command_port/2"},{"type":"function","title":"ExTermbox.ProcessManager.connect_socket/2","doc":"Connects to a Unix Domain Socket at the given path.","ref":"ExTermbox.ProcessManager.html#connect_socket/2"},{"type":"function","title":"ExTermbox.ProcessManager.send_socket/2","doc":"Uses :gen_tcp.send for sending data over the UDS connection established with :gen_tcp.connect.","ref":"ExTermbox.ProcessManager.html#send_socket/2"},{"type":"function","title":"ExTermbox.ProcessManager.spawn_port/3","doc":"Uses Port.open to start the C port process.\n\nOpts: [:binary, :exit_status, :nouse_stdio]\n- :binary - data is sent/received as binaries\n- :exit_status - receive a message when the port exits\n- :nouse_stdio - prevent port from interacting with group leader stdio","ref":"ExTermbox.ProcessManager.html#spawn_port/3"},{"type":"module","title":"ExTermbox.Protocol","doc":"Defines the protocol for communication between Elixir and the C Termbox helper.\nIncludes parsing/formatting functions.","ref":"ExTermbox.Protocol.html"},{"type":"function","title":"ExTermbox.Protocol.format_change_cell_command/5","doc":"","ref":"ExTermbox.Protocol.html#format_change_cell_command/5"},{"type":"function","title":"ExTermbox.Protocol.format_clear_command/0","doc":"","ref":"ExTermbox.Protocol.html#format_clear_command/0"},{"type":"function","title":"ExTermbox.Protocol.format_debug_send_event_command/8","doc":"","ref":"ExTermbox.Protocol.html#format_debug_send_event_command/8"},{"type":"function","title":"ExTermbox.Protocol.format_get_cell_command/2","doc":"","ref":"ExTermbox.Protocol.html#format_get_cell_command/2"},{"type":"function","title":"ExTermbox.Protocol.format_height_command/0","doc":"","ref":"ExTermbox.Protocol.html#format_height_command/0"},{"type":"function","title":"ExTermbox.Protocol.format_present_command/0","doc":"","ref":"ExTermbox.Protocol.html#format_present_command/0"},{"type":"function","title":"ExTermbox.Protocol.format_print_command/5","doc":"","ref":"ExTermbox.Protocol.html#format_print_command/5"},{"type":"function","title":"ExTermbox.Protocol.format_set_clear_attributes_command/2","doc":"","ref":"ExTermbox.Protocol.html#format_set_clear_attributes_command/2"},{"type":"function","title":"ExTermbox.Protocol.format_set_cursor_command/2","doc":"","ref":"ExTermbox.Protocol.html#format_set_cursor_command/2"},{"type":"function","title":"ExTermbox.Protocol.format_set_input_mode_command/1","doc":"","ref":"ExTermbox.Protocol.html#format_set_input_mode_command/1"},{"type":"function","title":"ExTermbox.Protocol.format_set_output_mode_command/1","doc":"","ref":"ExTermbox.Protocol.html#format_set_output_mode_command/1"},{"type":"function","title":"ExTermbox.Protocol.format_width_command/0","doc":"","ref":"ExTermbox.Protocol.html#format_width_command/0"},{"type":"function","title":"ExTermbox.Protocol.parse_event/1","doc":"","ref":"ExTermbox.Protocol.html#parse_event/1"},{"type":"function","title":"ExTermbox.Protocol.parse_socket_line/1","doc":"","ref":"ExTermbox.Protocol.html#parse_socket_line/1"},{"type":"extras","title":"Raxol Revived ExTermbox","doc":"# Raxol Revived ExTermbox\n\n[![Hex.pm](https://img.shields.io/hexpm/v/rrex_termbox.svg)](https://hex.pm/packages/rrex_termbox)\n[![Hexdocs.pm](https://img.shields.io/badge/api-hexdocs-brightgreen.svg)](https://hexdocs.pm/rrex_termbox)\n\nAn Elixir library for interacting with the terminal via the [termbox](https://github.com/nsf/termbox) C library.\n\n**Starting with version 1.1.0, this library no longer uses Elixir Native Implemented Functions (NIFs).** Instead, it manages a separate C helper process (`termbox_port`) and communicates with it using an Elixir Port for initialization and Unix Domain Sockets (UDS) for runtime commands and events. This approach provides enhanced stability and avoids potential NIF-related pitfalls.\n\nFor high-level, declarative terminal UIs in Elixir, see [raxol](https://github.com/Hydepwns/raxol) or its predecessor [Ratatouille](https://github.com/ndreynolds/ratatouille), which build on top of this library.\n\nFor the API Reference, see the `ExTermbox` module: [https://hexdocs.pm/rrex_termbox/ExTermbox.html](https://hexdocs.pm/rrex_termbox/ExTermbox.html).","ref":"readme.html"},{"type":"extras","title":"Getting Started - Raxol Revived ExTermbox","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Architecture (Port/UDS Based) - Raxol Revived ExTermbox","doc":"**Note:** If you previously used versions prior to 1.1.0, be aware that the underlying communication mechanism has changed significantly from NIFs to a Port/UDS system. The `ExTermbox.Bindings` module and NIF-based event polling have been removed. See the [Changelog](./CHANGELOG.md) for details.\n\nExTermbox starts and manages a C helper program (`termbox_port`). Communication happens as follows:\n\n1. **Initialization:** An Elixir Port is used briefly to exchange the path for a Unix Domain Socket (UDS).\n2. **Runtime:** All subsequent commands (like printing, setting cursor, changing cells) and events (like key presses, resizes) are sent over the UDS connection using a simple text-based protocol.\n\nThe public API is exposed primarily through the `ExTermbox` module.","ref":"readme.html#architecture-port-uds-based"},{"type":"extras","title":"Hello World - Raxol Revived ExTermbox","doc":"Let's go through a simple example.\nTo follow along, clone this repo and save the code below as an `.exs` file (e.g., `hello.exs`).\n\nThis repository makes use of [Git submodules](https://git-scm.com/book/en/v2/Git-Tools-Submodules), so make sure you include them in your clone. In recent versions of git, this can be accomplished by including the `--recursive` flag, e.g.\n\n```bash\n# Make sure to clone *this* repository recursively to include submodules\ngit clone --recursive https://github.com/Hydepwns/rrex_termbox.git\n```\n\nWhen the clone is complete, the `c_src/termbox/` directory should have files in it.\n\nYou can also create an\nElixir script in any Mix project with `rrex_termbox` in the dependencies list.\nLater, we'll run the example with `mix run  `.\n\n```elixir\n# hello.exs\ndefmodule HelloWorld do\n  alias ExTermbox\n\n  def run do\n    # Start ExTermbox, registering the current process to receive events\n    case ExTermbox.init(self()) do\n      :ok ->\n        IO.puts(\"ExTermbox initialized successfully.\")\n        # Clear the screen\n        :ok = ExTermbox.clear()\n\n        # Print \"Hello, World!\" at (0, 0) with default colors\n        :ok = ExTermbox.print(0, 0, :default, :default, \"Hello, World!\")\n\n        # Print \"(Press   to quit)\" at (0, 2)\n        :ok = ExTermbox.print(0, 2, :default, :default, \"(Press   to quit)\")\n\n        # Render the changes to the terminal\n        :ok = ExTermbox.present()\n\n        # Wait for the 'q' key event\n        wait_for_quit()\n\n        # Shut down ExTermbox\n        :ok = ExTermbox.shutdown()\n        IO.puts(\"ExTermbox shut down.\")\n\n      {:error, reason} ->\n        IO.inspect(reason, label: \"Error initializing ExTermbox\")\n    end\n  end\n\n  defp wait_for_quit do\n    receive do\n      # Events are sent as messages to the registered process\n      {:termbox_event, %{type: :key, key: :q}} ->\n        :quit\n      {:termbox_event, event} ->\n        # IO.inspect(event, label: \"Received event\") # Uncomment to see other events\n        wait_for_quit() # Wait for the next event\n      _other_message ->\n        # IO.inspect(other_message, label: \"Received other message\")\n        wait_for_quit() # Wait for the next event\n    after\n      10_000 -> IO.puts(\"Timeout waiting for 'q' key.\") # Add a timeout for safety\n    end\n  end\nend\n\nHelloWorld.run()\n\n```\n\nIn a real application, you'll likely want to integrate `ExTermbox` into an OTP application with a proper supervisor.\n\nThe `ExTermbox.print/5` function provides a simple way to display strings. For more control over individual cells (characters, foreground/background colors, attributes), use `ExTermbox.change_cell/5`. The `ExTermbox.width/0` and `ExTermbox.height/0` functions can be used to get the terminal dimensions.\n\nFinally, run the example like this:\n\n```bash\nmix run hello.exs\n```\n\nYou should see the text we rendered and be able to quit with 'q'.","ref":"readme.html#hello-world"},{"type":"extras","title":"Python Build Compatibility (Python 3.12+) - Raxol Revived ExTermbox","doc":"The version of the `termbox` C library bundled with `:rrex_termbox` uses an older version of the `waf` build system. This version of `waf` contained code (`import imp`) that is incompatible with Python 3.12 and newer, causing the C helper compilation to fail if a modern Python version is your system default.\n\nThis fork includes a small patch directly within the bundled `waf` scripts (`c_src/termbox/.waf3-2.0.14-e67604cd8962dbdaf7c93e0d7470ef5b/waflib/Context.py`) to replace the incompatible code with its modern equivalent (`importlib`).\n\nWith this patch, `:rrex_termbox` should compile successfully using Python 3.12+ without requiring manual intervention or downgrading Python.\n\nIf you encounter build issues related to Python or `waf`, please ensure you are using a version of this library that includes this fix.","ref":"readme.html#python-build-compatibility-python-3-12"},{"type":"extras","title":"Installation - Raxol Revived ExTermbox","doc":"Add `:rrex_termbox` as a dependency in your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:rrex_termbox, \"~> 1.1.0\"}\n  ]\nend\n```\n\nThe Hex package bundles a compatible version of termbox. You will need standard C build tools (like `gcc` or `clang`, often part of `build-essential` or Xcode Command Line Tools) installed on your system.\n\nMix compile hooks automatically build the `termbox_port` C helper executable needed by the library. This should happen the first time you build :rrex_termbox (e.g., via `mix deps.get` followed by `mix deps.compile` or simply `mix compile`).\n\nThe build has been tested on macOS and some Linux distributions. Please open an issue if you encounter build problems.","ref":"readme.html#installation"},{"type":"extras","title":"Using the Source Directly - Raxol Revived ExTermbox","doc":"To try out the master branch, first clone the repo:\n\n```bash\n# Make sure to clone *this* repository recursively to include submodules\ngit clone --recurse-submodules  \ncd rrex_termbox # Assuming the directory name matches the repo\n```\n\nThe `--recurse-submodules` flag (`--recursive` before Git 2.13) is necessary in\norder to additionally clone the termbox source code, which is required to\nbuild the C helper program.\n\nNext, fetch the deps:\n\n```\nmix deps.get\n```\n\nFinally, try out the included event viewer application:\n\n```\nmix run examples/event_viewer.exs\n```\n\nIf you see the application drawn and can trigger events, you're good to go. Use\n'q' to quit the examples.","ref":"readme.html#using-the-source-directly"},{"type":"extras","title":"Distribution - Raxol Revived ExTermbox","doc":"Building a standalone executable for applications using `:rrex_termbox` requires some special consideration due to the included C helper program (`termbox_port`) which needs to be packaged alongside your Elixir application.\n\nStandard Elixir tools like `escript` are **not** suitable because they do not correctly package the necessary helper executable located in the `priv/` directory after compilation.\n\nThe recommended approach for creating distributable releases is to use **[Distillery](https://github.com/bitwalker/distillery)** or the built-in Elixir **[Releases](https://hexdocs.pm/mix/Mix.Tasks.Release.html)**. These tools are designed to handle external programs and assets correctly. They will package your application, the Erlang Runtime System (ERTS), and the compiled `termbox_port` helper into a self-contained bundle.\n\nConsult the documentation for Distillery or Elixir Releases for specific instructions on configuring your project for release builds, ensuring the `priv/termbox_port` executable is included in the release package.","ref":"readme.html#distribution"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}